@@ -1,7 +1,7 @@
-#ifndef __CONDITIONALDIALOG_H__
-#define __CONDITIONALDIALOG_H__
+#ifndef FILEZILLA_INTERFACE_CONDITIONALDIALOG_HEADER
+#define FILEZILLA_INTERFACE_CONDITIONALDIALOG_HEADER
 
-class CConditionalDialog : public wxDialog
+class CConditionalDialog final : public wxDialog
 {
 public:
 	enum Modes
@@ -20,7 +20,8 @@ public:
 		confirm_preserve_timestamps,
 		compare_treeviewmismatch,
 		compare_changesorting,
-		many_selected_for_edit
+		many_selected_for_edit,
+		quick_search
 	};
 
 	CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked = false);
@@ -38,4 +39,4 @@ protected:
 	void OnButton(wxCommandEvent& event);
 };
 
-#endif //__CONDITIONALDIALOG_H__
+#endif
