@@ -9,6 +9,8 @@
 
 #include <wx/dcclient.h>
 
+#include <algorithm>
+
 static const int statbarWidths[3] = {
 	-3, 0, 25
 };
@@ -29,8 +31,9 @@ wxStatusBarEx::wxStatusBarEx(wxTopLevelWindow* pParent)
 #ifdef __WXMSW__
 	m_parentWasMaximized = false;
 
-	if (GetLayoutDirection() != wxLayout_RightToLeft)
+	if (GetLayoutDirection() != wxLayout_RightToLeft) {
 		SetDoubleBuffered(true);
+	}
 #endif
 }
 
@@ -39,7 +42,7 @@ wxStatusBarEx::~wxStatusBarEx()
 	delete [] m_columnWidths;
 }
 
-void wxStatusBarEx::SetFieldsCount(int number /*=1*/, const int* widths /*=NULL*/)
+void wxStatusBarEx::SetFieldsCount(int number, const int* widths)
 {
 	wxASSERT(number > 0);
 
@@ -47,28 +50,28 @@ void wxStatusBarEx::SetFieldsCount(int number /*=1*/, const int* widths /*=NULL*
 	int* oldWidths = m_columnWidths;
 
 	m_columnWidths = new int[number];
-	if (!widths)
-	{
-		if (oldWidths)
-		{
-			const int min = wxMin(oldCount, number);
-			for (int i = 0; i < min; i++)
+	if (!widths) {
+		if (oldWidths) {
+			const int min = std::min(oldCount, number);
+			for (int i = 0; i < min; ++i) {
 				m_columnWidths[i] = oldWidths[i];
-			for (int i = min; i < number; i++)
+			}
+			for (int i = min; i < number; ++i) {
 				m_columnWidths[i] = -1;
+			}
 			delete [] oldWidths;
 		}
-		else
-		{
-			for (int i = 0; i < number; i++)
+		else {
+			for (int i = 0; i < number; ++i) {
 				m_columnWidths[i] = -1;
+			}
 		}
 	}
-	else
-	{
+	else {
 		delete [] oldWidths;
-		for (int i = 0; i < number; i++)
+		for (int i = 0; i < number; ++i) {
 			m_columnWidths[i] = widths[i];
+		}
 
 		FixupFieldWidth(number - 1);
 	}
@@ -80,8 +83,9 @@ void wxStatusBarEx::SetStatusWidths(int n, const int *widths)
 {
 	wxASSERT(n == GetFieldsCount());
 	wxASSERT(widths);
-	for (int i = 0; i < n; i++)
+	for (int i = 0; i < n; ++i) {
 		m_columnWidths[i] = widths[i];
+	}
 
 	FixupFieldWidth(n - 1);
 
@@ -90,20 +94,22 @@ void wxStatusBarEx::SetStatusWidths(int n, const int *widths)
 
 void wxStatusBarEx::FixupFieldWidth(int field)
 {
-	if (field != GetFieldsCount() - 1)
+	if (field != GetFieldsCount() - 1) {
 		return;
+	}
 
 #if __WXGTK20__
 	// Gripper overlaps last all the time
-	if (m_columnWidths[field] > 0)
+	if (m_columnWidths[field] > 0) {
 		m_columnWidths[field] += 15;
+	}
 #endif
 }
 
 void wxStatusBarEx::SetFieldWidth(int field, int width)
 {
 	field = GetFieldIndex(field);
-	if( field < 0 ) {
+	if (field < 0) {
 		return;
 	}
 
@@ -129,18 +135,18 @@ void wxStatusBarEx::OnSize(wxSizeEvent&)
 {
 #ifdef __WXMSW__
 	const int count = GetFieldsCount();
-	if (count && m_columnWidths && m_columnWidths[count - 1] > 0)
-	{
+	if (count && m_columnWidths && m_columnWidths[count - 1] > 0) {
 		// No sizegrip on maximized windows
 		bool isMaximized = m_pParent->IsMaximized();
-		if (isMaximized != m_parentWasMaximized)
-		{
+		if (isMaximized != m_parentWasMaximized) {
 			m_parentWasMaximized = isMaximized;
 
-			if (isMaximized)
+			if (isMaximized) {
 				m_columnWidths[count - 1] -= 16;
-			else
+			}
+			else {
 				m_columnWidths[count - 1] += 16;
+			}
 
 			wxStatusBar::SetStatusWidths(count, m_columnWidths);
 			Refresh();
@@ -189,19 +195,21 @@ void CWidgetsStatusBar::OnSize(wxSizeEvent& event)
 {
 	wxStatusBarEx::OnSize(event);
 
-	for (int i = 0; i < GetFieldsCount(); i++)
+	for (int i = 0; i < GetFieldsCount(); ++i) {
 		PositionChildren(i);
+	}
 
 #ifdef __WXMSW__
-	if (GetLayoutDirection() != wxLayout_RightToLeft)
+	if (GetLayoutDirection() != wxLayout_RightToLeft) {
 		Update();
+	}
 #endif
 }
 
 bool CWidgetsStatusBar::AddField(int field, int idx, wxWindow* pChild)
 {
 	field = GetFieldIndex(field);
-	if( field < 0 ) {
+	if (field < 0) {
 		return false;
 	}
 
@@ -234,17 +242,16 @@ void CWidgetsStatusBar::PositionChildren(int field)
 	int offset = 2;
 
 #ifndef __WXMSW__
-	if (field + 1 == GetFieldsCount())
-	{
+	if (field + 1 == GetFieldsCount()) {
 		rect.SetWidth(m_columnWidths[field]);
 		offset += 5 + GetGripperWidth();
 	}
 #endif
 
-	for (auto iter = m_children.begin(); iter != m_children.end(); ++iter)
-	{
-		if (iter->second.field != field)
+	for (auto iter = m_children.begin(); iter != m_children.end(); ++iter) {
+		if (iter->second.field != field) {
 			continue;
+		}
 
 		const wxSize size = iter->second.pChild->GetSize();
 		int position = rect.GetRight() - size.x - offset;
@@ -258,8 +265,9 @@ void CWidgetsStatusBar::PositionChildren(int field)
 void CWidgetsStatusBar::SetFieldWidth(int field, int width)
 {
 	wxStatusBarEx::SetFieldWidth(field, width);
-	for (int i = 0; i < GetFieldsCount(); i++)
+	for (int i = 0; i < GetFieldsCount(); ++i) {
 		PositionChildren(i);
+	}
 }
 
 class CIndicator : public wxStaticBitmap
@@ -297,7 +305,7 @@ EVT_TIMER(wxID_ANY, CStatusBar::OnTimer)
 END_EVENT_TABLE()
 
 CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
-	: CWidgetsStatusBar(pParent), CStateEventHandler(0)
+	: CWidgetsStatusBar(pParent)
 {
 	// Speedlimits
 	RegisterOption(OPTION_SPEEDLIMIT_ENABLE);
@@ -312,11 +320,11 @@ CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
 	RegisterOption(OPTION_ASCIIBINARY);
 
 	// Reload icons
-	RegisterOption(OPTION_THEME);
+	RegisterOption(OPTION_ICONS_THEME);
 
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true, false);
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_ENCRYPTION, true, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_ENCRYPTION, true);
 
 	const int count = 3;
 	SetFieldsCount(count);
@@ -371,12 +379,13 @@ void CStatusBar::DoDisplayQueueSize()
 
 void CStatusBar::DisplayDataType()
 {
-	const CServer* pServer = 0;
-	const CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (pState)
-		pServer = pState->GetServer();
+	ServerWithCredentials server;
+	CState const* pState = CContextManager::Get()->GetCurrentContext();
+	if (pState) {
+		server = pState->GetServer();
+	}
 
-	if (!pServer || !CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol())) {
+	if (!server || !CServer::ProtocolHasDataTypeConcept(server.server.GetProtocol())) {
 		if (m_pDataTypeIndicator) {
 			RemoveField(widget_datatype);
 			m_pDataTypeIndicator->Destroy();
@@ -401,13 +410,14 @@ void CStatusBar::DisplayDataType()
 			desc = _("Current transfer type is set to automatic detection.");
 		}
 
-		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+		wxBitmap bmp = CThemeProvider::Get()->CreateBitmap(name, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
 		if (!m_pDataTypeIndicator) {
 			m_pDataTypeIndicator = new CIndicator(this, bmp);
 			AddField(0, widget_datatype, m_pDataTypeIndicator);
 		}
-		else
+		else {
 			m_pDataTypeIndicator->SetBitmap(bmp);
+		}
 		m_pDataTypeIndicator->SetToolTip(desc);
 	}
 }
@@ -420,11 +430,11 @@ void CStatusBar::MeasureQueueSizeWidth()
 	wxSize s = dc.GetTextExtent(_("Queue: empty"));
 
 	wxString tmp = _T(">8888");
-	if (m_sizeFormatDecimalPlaces)
-	{
+	if (m_sizeFormatDecimalPlaces) {
 		tmp += _T(".");
-		for (int i = 0; i < m_sizeFormatDecimalPlaces; i++)
+		for (int i = 0; i < m_sizeFormatDecimalPlaces; ++i) {
 			tmp += _T("8");
+		}
 	}
 	s.IncTo(dc.GetTextExtent(wxString::Format(_("Queue: %s MiB"), tmp)));
 
@@ -433,18 +443,19 @@ void CStatusBar::MeasureQueueSizeWidth()
 
 void CStatusBar::DisplayEncrypted()
 {
-	const CServer* pServer = 0;
-	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (pState)
-		pServer = pState->GetServer();
+	ServerWithCredentials server;
+	CState *const pState = CContextManager::Get()->GetCurrentContext();
+	if (pState) {
+		server = pState->GetServer();
+	}
 
 	bool encrypted = false;
-	if (pServer) {
+	if (server) {
 		CCertificateNotification* info;
-		if (pServer->GetProtocol() == FTPS || pServer->GetProtocol() == FTPES || pServer->GetProtocol() == SFTP) {
+		if (server.server.GetProtocol() == FTPS || server.server.GetProtocol() == FTPES || server.server.GetProtocol() == SFTP) {
 			encrypted = true;
 		}
-		else if (pServer->GetProtocol() == FTP && pState->GetSecurityInfo(info)) {
+		else if (server.server.GetProtocol() == FTP && pState->GetSecurityInfo(info)) {
 			encrypted = true;
 		}
 	}
@@ -457,14 +468,15 @@ void CStatusBar::DisplayEncrypted()
 		}
 	}
 	else {
-		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER,  CThemeProvider::GetIconSize(iconSizeSmall));
+		wxBitmap bmp = CThemeProvider::Get()->CreateBitmap(_T("ART_LOCK"), wxART_OTHER,  CThemeProvider::GetIconSize(iconSizeSmall));
 		if (!m_pEncryptionIndicator) {
 			m_pEncryptionIndicator = new CIndicator(this, bmp);
 			AddField(0, widget_encryption, m_pEncryptionIndicator);
 			m_pEncryptionIndicator->SetToolTip(_("The connection is encrypted. Click icon for details."));
 		}
-		else
+		else {
 			m_pEncryptionIndicator->SetBitmap(bmp);
+		}
 	}
 }
 
@@ -491,7 +503,8 @@ void CStatusBar::OnHandleLeftClick(wxWindow* pWnd)
 		CCertificateNotification *pCertificateNotification = 0;
 		CSftpEncryptionNotification *pSftpEncryptionNotification = 0;
 		if (pState->GetSecurityInfo(pCertificateNotification)) {
-			CVerifyCertDialog dlg;
+			CertStore certStore;
+			CVerifyCertDialog dlg(certStore);
 			dlg.ShowVerificationDialog(*pCertificateNotification, true);
 		}
 		else if (pState->GetSecurityInfo(pSftpEncryptionNotification)) {
@@ -560,7 +573,7 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 {
 	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
 
-	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+	wxBitmap bmp = CThemeProvider::Get()->CreateBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
 	if (!bmp.Ok()) {
 		return;
 	}
@@ -571,29 +584,38 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 	if (!enable || (!downloadLimit && !uploadLimit)) {
 		wxImage img = bmp.ConvertToImage();
 		img = img.ConvertToGreyscale();
+#ifdef __WXMAC__
+		bmp = wxBitmap(img, -1, bmp.GetScaleFactor());
+#else
 		bmp = wxBitmap(img);
+#endif
 		tooltip = _("Speed limits are disabled, click to change.");
 	}
 	else {
 		tooltip = _("Speed limits are enabled, click to change.");
 		tooltip += _T("\n");
-		if (downloadLimit)
+		if (downloadLimit) {
 			tooltip += wxString::Format(_("Download limit: %s/s"), CSizeFormat::FormatUnit(downloadLimit, CSizeFormat::kilo));
-		else
+		}
+		else {
 			tooltip += _("Download limit: none");
+		}
 		tooltip += _T("\n");
-		if (uploadLimit)
+		if (uploadLimit) {
 			tooltip += wxString::Format(_("Upload limit: %s/s"), CSizeFormat::FormatUnit(uploadLimit, CSizeFormat::kilo));
-		else
+		}
+		else {
 			tooltip += _("Upload limit: none");
+		}
 	}
 
 	if (!m_pSpeedLimitsIndicator) {
 		m_pSpeedLimitsIndicator = new CIndicator(this, bmp);
 		AddField(0, widget_speedlimit, m_pSpeedLimitsIndicator);
 	}
-	else
+	else {
 		m_pSpeedLimitsIndicator->SetBitmap(bmp);
+	}
 	m_pSpeedLimitsIndicator->SetToolTip(tooltip);
 }
 
@@ -608,14 +630,14 @@ void CStatusBar::OnOptionsChanged(changed_options_t const& options)
 	if (options.test(OPTION_ASCIIBINARY)) {
 		DisplayDataType();
 	}
-	if (options.test(OPTION_THEME)) {
+	if (options.test(OPTION_ICONS_THEME)) {
 		DisplayDataType();
 		UpdateSpeedLimitsIcon();
 		DisplayEncrypted();
 	}
 }
 
-void CStatusBar::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
+void CStatusBar::OnStateChange(CState*, t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT) {
 		DisplayDataType();
@@ -631,18 +653,18 @@ void CStatusBar::OnSpeedLimitsEnable(wxCommandEvent&)
 	int downloadlimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
 	int uploadlimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
 	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) == 0;
-	if (enable)
-	{
-		if (!downloadlimit && !uploadlimit)
-		{
+	if (enable) {
+		if (!downloadlimit && !uploadlimit) {
 			CSpeedLimitsDialog dlg;
 			dlg.Run(m_pParent);
 		}
-		else
+		else {
 			COptions::Get()->SetOption(OPTION_SPEEDLIMIT_ENABLE, 1);
+		}
 	}
-	else
+	else {
 		COptions::Get()->SetOption(OPTION_SPEEDLIMIT_ENABLE, 0);
+	}
 }
 
 void CStatusBar::OnSpeedLimitsConfigure(wxCommandEvent&)
