@@ -7,15 +7,205 @@
 #include "timeformatting.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/iputils.hpp>
+
 #include <wx/scrolwin.h>
 #include <wx/statbox.h>
 #include <wx/tokenzr.h>
 
-CVerifyCertDialog::CVerifyCertDialog()
+CertStore::CertStore()
 	: m_xmlFile(wxGetApp().GetSettingsFile(_T("trustedcerts")))
 {
 }
 
+bool CertStore::IsTrusted(CCertificateNotification const& notification)
+{
+	if (notification.GetAlgorithmWarnings() != 0) {
+		// These certs are never trusted.
+		return false;
+	}
+
+	LoadTrustedCerts();
+
+	CCertificate cert = notification.GetCertificates()[0];
+
+	return IsTrusted(notification.GetHost(), notification.GetPort(), cert.GetRawData(), false, !notification.MismatchedHostname());
+}
+
+bool CertStore::DoIsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<CertStore::t_certData> const& trustedCerts, bool allowSans)
+{
+	if (!data.size()) {
+		return false;
+	}
+
+	bool const dnsname = fz::get_address_type(host) == fz::address_type::unknown;
+	
+	for (auto const& cert : trustedCerts) {
+		if (port != cert.port) {
+			continue;
+		}
+
+		if (cert.data != data) {
+			continue;
+		}
+
+		if (host != cert.host) {
+			if (!dnsname || !allowSans || !cert.trustSans) {
+				continue;
+			}
+		}
+
+		return true;
+	}
+
+	return false;
+}
+
+bool CertStore::IsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly, bool allowSans)
+{
+	bool trusted = DoIsTrusted(host, port, data, trustedCerts_, allowSans);
+	if (!trusted && !permanentOnly) {
+		trusted = DoIsTrusted(host, port, data, sessionTrustedCerts_, allowSans);
+	}
+
+	return trusted;
+}
+
+void CertStore::LoadTrustedCerts()
+{
+	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
+	if (!m_xmlFile.Modified()) {
+		return;
+	}
+
+	auto element = m_xmlFile.Load();
+	if (!element) {
+		return;
+	}
+
+	trustedCerts_.clear();
+
+	if (!(element = element.child("TrustedCerts"))) {
+		return;
+	}
+
+	bool modified = false;
+
+	auto const processEntry = [&](pugi::xml_node const& cert)
+	{
+		std::wstring value = GetTextElement(cert, "Data");
+
+		pugi::xml_node remove;
+
+		t_certData data;
+		data.data = fz::hex_decode(value);
+		if (data.data.empty()) {
+			return false;
+		}
+
+		data.host = GetTextElement(cert, "Host");
+		data.port = GetTextElementInt(cert, "Port");
+		if (data.host.empty() || data.port < 1 || data.port > 65535) {
+			return false;
+		}
+
+		fz::datetime const now = fz::datetime::now();
+		int64_t activationTime = GetTextElementInt(cert, "ActivationTime", 0);
+		if (activationTime == 0 || activationTime > now.get_time_t()) {
+			return false;
+		}
+
+		int64_t expirationTime = GetTextElementInt(cert, "ExpirationTime", 0);
+		if (expirationTime == 0 || expirationTime < now.get_time_t()) {
+			return false;
+		}
+
+		data.trustSans = GetTextElementBool(cert, "TrustSANs");
+
+		// Weed out duplicates
+		if (IsTrusted(data.host, data.port, data.data, true, false)) {
+			return false;
+		}
+
+		trustedCerts_.emplace_back(std::move(data));
+
+		return true;
+	};
+
+	auto cert = element.child("Certificate");
+	while (cert) {
+
+		auto nextCert = cert.next_sibling("Certificate");
+		if (!processEntry(cert)) {
+			modified = true;
+			element.remove_child(cert);
+		}
+		cert = nextCert;
+	}
+
+	if (modified) {
+		m_xmlFile.Save(false);
+	}
+}
+
+void CertStore::SetTrusted(CCertificateNotification const& notification, bool permanent, bool trustAllHostnames)
+{
+	const CCertificate certificate = notification.GetCertificates()[0];
+
+	t_certData cert;
+	cert.host = notification.GetHost();
+	cert.port = notification.GetPort();
+	cert.data = certificate.GetRawData();
+
+	if (trustAllHostnames) {
+		cert.trustSans = true;
+	}
+
+	if (!permanent) {
+		t_certData cert;
+		sessionTrustedCerts_.emplace_back(std::move(cert));
+
+		return;
+	}
+
+	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
+	LoadTrustedCerts();
+
+	if (IsTrusted(cert.host, cert.port, cert.data, true, false)) {
+		return;
+	}
+
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 2) {
+		auto element = m_xmlFile.GetElement();
+		if (element) {
+			auto certs = element.child("TrustedCerts");
+			if (!certs) {
+				certs = element.append_child("TrustedCerts");
+			}
+
+			auto xCert = certs.append_child("Certificate");
+			AddTextElementUtf8(xCert, "Data", fz::hex_encode<std::string>(cert.data));
+			AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.GetActivationTime().get_time_t()));
+			AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.GetExpirationTime().get_time_t()));
+			AddTextElement(xCert, "Host", cert.host);
+			AddTextElement(xCert, "Port", cert.port);
+			AddTextElement(xCert, "TrustSANs", cert.trustSans ? L"1" : L"0");
+
+			m_xmlFile.Save(true);
+		}
+	}
+	trustedCerts_.emplace_back(std::move(cert));
+}
+
+
+
+
+CVerifyCertDialog::CVerifyCertDialog(CertStore & certStore)
+	: certStore_(certStore)
+{
+}
+
+
 bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 {
 	bool warning = false;
@@ -74,7 +264,7 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	if (!altNames.empty()) {
 		wxString str;
 		for (auto const& altName : altNames) {
-			str += altName + _T("\n");
+			str += altName.name + _T("\n");
 		}
 		str.RemoveLast();
 		m_pSubjectSizer->Add(new wxStaticText(subjectPanel, wxID_ANY, wxPLURAL("Alternative name:", "Alternative names:", altNames.size())));
@@ -117,10 +307,8 @@ bool CVerifyCertDialog::DisplayAlgorithm(int controlId, wxString name, bool inse
 	return insecure;
 }
 
-void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly /*=false*/)
+void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly)
 {
-	LoadTrustedCerts();
-
 	m_pDlg = new wxDialogEx;
 	if (!m_pDlg->Load(0, _T("ID_VERIFYCERT"))) {
 		wxBell();
@@ -133,6 +321,7 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 		xrc_call(*m_pDlg, "ID_DESC", &wxWindow::Hide);
 		xrc_call(*m_pDlg, "ID_ALWAYS_DESC", &wxWindow::Hide);
 		xrc_call(*m_pDlg, "ID_ALWAYS", &wxWindow::Hide);
+		xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxWindow::Hide);
 		xrc_call(*m_pDlg, "wxID_CANCEL", &wxWindow::Hide);
 		m_pDlg->SetTitle(_T("Certificate details"));
 	}
@@ -200,6 +389,10 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 		XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
 	}
 
+	bool const dnsname = fz::get_address_type(notification.GetHost()) == fz::address_type::unknown;
+	bool const sanTrustAllowed = !warning && dnsname && !notification.MismatchedHostname();
+		XRCCTRL(*m_pDlg, "ID_TRUST_SANS", wxCheckBox)->Enable(sanTrustAllowed);
+
 	m_pDlg->GetSizer()->Fit(m_pDlg);
 	m_pDlg->GetSizer()->SetSizeHints(m_pDlg);
 
@@ -207,21 +400,12 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 
 	if (!displayOnly) {
 		if (res == wxID_OK) {
-			wxASSERT(!IsTrusted(notification));
-
 			notification.m_trusted = true;
 
 			if (!notification.GetAlgorithmWarnings()) {
-				if (!warning && XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->GetValue()) {
-					SetPermanentlyTrusted(notification);
-				}
-				else {
-					t_certData cert;
-					cert.host = notification.GetHost();
-					cert.port = notification.GetPort();
-					cert.data = m_certificates[0].GetRawData();
-					m_sessionTrustedCerts.emplace_back(std::move(cert));
-				}
+				bool trustSANs = sanTrustAllowed && xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxCheckBox::GetValue);
+				bool permanent = !warning && xrc_call(*m_pDlg, "ID_ALWAYS", &wxCheckBox::GetValue);
+				certStore_.SetTrusted(notification, permanent, trustSANs);
 			}
 		}
 		else {
@@ -320,161 +504,6 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxWindow* parent, std::list<wxString>&
 	}
 }
 
-bool CVerifyCertDialog::IsTrusted(CCertificateNotification const& notification)
-{
-	if (notification.GetAlgorithmWarnings() != 0) {
-		// These certs are never trusted.
-		return false;
-	}
-
-	LoadTrustedCerts();
-
-	CCertificate cert = notification.GetCertificates()[0];
-
-	return IsTrusted(notification.GetHost(), notification.GetPort(), cert.GetRawData(), false);
-}
-
-bool CVerifyCertDialog::DoIsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<CVerifyCertDialog::t_certData> const& trustedCerts)
-{
-	if (!data.size()) {
-		return false;
-	}
-
-	for (auto const& cert : trustedCerts) {
-		if (host != cert.host) {
-			continue;
-		}
-
-		if (port != cert.port) {
-			continue;
-		}
-
-		if (cert.data == data) {
-			return true;
-		}
-	}
-
-	return false;
-}
-
-bool CVerifyCertDialog::IsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly)
-{
-	bool trusted = DoIsTrusted(host, port, data, m_trustedCerts);
-	if (!trusted && !permanentOnly) {
-		trusted = DoIsTrusted(host, port, data, m_sessionTrustedCerts);
-	}
-
-	return trusted;
-}
-
-void CVerifyCertDialog::LoadTrustedCerts()
-{
-	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
-	if (!m_xmlFile.Modified()) {
-		return;
-	}
-
-	auto element = m_xmlFile.Load();
-	if (!element) {
-		return;
-	}
-
-	m_trustedCerts.clear();
-
-	if (!(element = element.child("TrustedCerts"))) {
-		return;
-	}
-
-	bool modified = false;
-
-	auto cert = element.child("Certificate");
-	while (cert) {
-		std::wstring value = GetTextElement(cert, "Data");
-
-		pugi::xml_node remove;
-
-		t_certData data;
-		data.data = fz::hex_decode(value);
-		if (data.data.empty()) {
-			remove = cert;
-		}
-
-		data.host = GetTextElement(cert, "Host");
-		data.port = GetTextElementInt(cert, "Port");
-		if (data.host.empty() || data.port < 1 || data.port > 65535) {
-			remove = cert;
-		}
-
-		fz::datetime const now = fz::datetime::now();
-		int64_t activationTime = GetTextElementInt(cert, "ActivationTime", 0);
-		if (activationTime == 0 || activationTime > now.get_time_t()) {
-			remove = cert;
-		}
-
-		int64_t expirationTime = GetTextElementInt(cert, "ExpirationTime", 0);
-		if (expirationTime == 0 || expirationTime < now.get_time_t()) {
-			remove = cert;
-		}
-
-		// Weed out duplicates
-		if (IsTrusted(data.host, data.port, data.data, true)) {
-			remove = cert;
-		}
-
-		if (!remove) {
-			m_trustedCerts.emplace_back(std::move(data));
-		}
-
-		cert = cert.next_sibling("Certificate");
-
-		if (remove) {
-			modified = true;
-			element.remove_child(remove);
-		}
-	}
-
-	if (modified) {
-		m_xmlFile.Save(false);
-	}
-}
-
-void CVerifyCertDialog::SetPermanentlyTrusted(CCertificateNotification const& notification)
-{
-	const CCertificate certificate = notification.GetCertificates()[0];
-
-	t_certData cert;
-	cert.host = notification.GetHost();
-	cert.port = notification.GetPort();
-	cert.data = certificate.GetRawData();
-
-	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
-	LoadTrustedCerts();
-
-	if (IsTrusted(cert.host, cert.port, cert.data, true))	{
-		return;
-	}
-
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 2) {
-		auto element = m_xmlFile.GetElement();
-		if (element) {
-			auto certs = element.child("TrustedCerts");
-			if (!certs) {
-				certs = element.append_child("TrustedCerts");
-			}
-
-			auto xCert = certs.append_child("Certificate");
-			AddTextElementUtf8(xCert, "Data", fz::hex_encode<std::string>(cert.data));
-			AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.GetActivationTime().get_time_t()));
-			AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.GetExpirationTime().get_time_t()));
-			AddTextElement(xCert, "Host", cert.host);
-			AddTextElement(xCert, "Port", cert.port);
-
-			m_xmlFile.Save(true);
-		}
-	}
-	m_trustedCerts.emplace_back(std::move(cert));
-}
-
 wxString CVerifyCertDialog::DecodeValue(const wxString& value)
 {
 	// Decodes string in hex notation
